@@ -1,3 +1,4 @@
+# coding: utf-8
 import hashlib
 import warnings
 import logging
@@ -43,6 +44,8 @@ from urllib3.packages import six
 
 from . import clear_warnings
 
+from test import onlyPy3, onlyPy2, onlyBrotlipy, notBrotlipy
+
 # This number represents a time in seconds, it doesn't mean anything in
 # isolation. Setting to a high-ish value to avoid conflicts with the smaller
 # numbers used for timeouts
@@ -128,12 +131,24 @@ class TestUtil(object):
         with pytest.raises(LocationParseError):
             get_host(location)
 
+    @pytest.mark.parametrize('url', [
+        'http://user\\@google.com',
+        'http://google\\.com',
+        'user\\@google.com',
+        'http://google.com#fragment#',
+        'http://user@user@google.com/',
+    ])
+    def test_invalid_url(self, url):
+        with pytest.raises(LocationParseError):
+            parse_url(url)
+
     @pytest.mark.parametrize('url, expected_normalized_url', [
         ('HTTP://GOOGLE.COM/MAIL/', 'http://google.com/MAIL/'),
         ('HTTP://JeremyCline:Hunter2@Example.com:8080/',
          'http://JeremyCline:Hunter2@example.com:8080/'),
         ('HTTPS://Example.Com/?Key=Value', 'https://example.com/?Key=Value'),
         ('Https://Example.Com/#Fragment', 'https://example.com/#Fragment'),
+        ('[::Ff%etH0%Ff]/%ab%Af', '[::ff%25etH0%Ff]/%AB%AF'),
     ])
     def test_parse_url_normalization(self, url, expected_normalized_url):
         """Assert parse_url normalizes the scheme/host, and only the scheme/host"""
@@ -152,12 +167,14 @@ class TestUtil(object):
         # Path/query/fragment
         ('', Url()),
         ('/', Url(path='/')),
-        ('#?/!google.com/?foo#bar', Url(path='', fragment='?/!google.com/?foo#bar')),
+        ('#?/!google.com/?foo', Url(path='', fragment='?/!google.com/?foo')),
         ('/foo', Url(path='/foo')),
         ('/foo?bar=baz', Url(path='/foo', query='bar=baz')),
         ('/foo?bar=baz#banana?apple/orange', Url(path='/foo',
                                                  query='bar=baz',
                                                  fragment='banana?apple/orange')),
+        ('/redirect?target=http://localhost:61020/', Url(path='redirect',
+                                                         query='target=http://localhost:61020/')),
 
         # Port
         ('http://google.com/', Url('http', host='google.com', path='/')),
@@ -167,11 +184,20 @@ class TestUtil(object):
         # Auth
         ('http://foo:bar@localhost/', Url('http', auth='foo:bar', host='localhost', path='/')),
         ('http://foo@localhost/', Url('http', auth='foo', host='localhost', path='/')),
-        ('http://foo:bar@baz@localhost/', Url('http',
-                                              auth='foo:bar@baz',
-                                              host='localhost',
-                                              path='/')),
-        ('http://@', Url('http', host=None, auth=''))
+        ('http://foo:bar@localhost/', Url('http',
+                                          auth='foo:bar',
+                                          host='localhost',
+                                          path='/')),
+
+        # Unicode type (Python 2.x)
+        (u'http://foo:bar@localhost/', Url(u'http',
+                                           auth=u'foo:bar',
+                                           host=u'localhost',
+                                           path=u'/')),
+        ('http://foo:bar@localhost/', Url('http',
+                                          auth='foo:bar',
+                                          host='localhost',
+                                          path='/')),
     ]
 
     non_round_tripping_parse_url_host_map = [
@@ -179,9 +205,16 @@ class TestUtil(object):
         ('?', Url(path='', query='')),
         ('#', Url(path='', fragment='')),
 
+        # Path normalization
+        ('/abc/../def', Url(path="/def")),
+
         # Empty Port
         ('http://google.com:', Url('http', host='google.com')),
         ('http://google.com:/', Url('http', host='google.com', path='/')),
+
+        # Uppercase IRI
+        (u'http://Königsgäßchen.de/straße',
+         Url('http', host='xn--knigsgchen-b4a3dun.de', path='/stra%C3%9Fe'))
     ]
 
     @pytest.mark.parametrize(
@@ -196,10 +229,31 @@ class TestUtil(object):
     def test_unparse_url(self, url, expected_url):
         assert url == expected_url.url
 
+    @pytest.mark.parametrize(
+        ['url', 'expected_url'],
+        [
+            # RFC 3986 5.2.4
+            ('/abc/../def', Url(path="/def")),
+            ('/..', Url(path="/")),
+            ('/./abc/./def/', Url(path='/abc/def/')),
+            ('/.', Url(path='/')),
+            ('/./', Url(path='/')),
+            ('/abc/./.././d/././e/.././f/./../../ghi', Url(path='/ghi'))
+        ]
+    )
+    def test_parse_and_normalize_url_paths(self, url, expected_url):
+        actual_url = parse_url(url)
+        assert actual_url == expected_url
+        assert actual_url.url == expected_url.url
+
     def test_parse_url_invalid_IPv6(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(LocationParseError):
             parse_url('[::1')
 
+    def test_parse_url_negative_port(self):
+        with pytest.raises(LocationParseError):
+            parse_url("https://www.google.com:-80/")
+
     def test_Url_str(self):
         U = Url('http', host='google.com')
         assert str(U) == U.url
@@ -232,15 +286,103 @@ class TestUtil(object):
     def test_netloc(self, url, expected_netloc):
         assert parse_url(url).netloc == expected_netloc
 
+    url_vulnerabilities = [
+        # urlparse doesn't follow RFC 3986 Section 3.2
+        ("http://google.com#@evil.com/", Url("http",
+                                             host="google.com",
+                                             path="",
+                                             fragment="@evil.com/")),
+
+        # CVE-2016-5699
+        ("http://127.0.0.1%0d%0aConnection%3a%20keep-alive",
+         Url("http", host="127.0.0.1%0d%0aconnection%3a%20keep-alive")),
+
+        # NodeJS unicode -> double dot
+        (u"http://google.com/\uff2e\uff2e/abc", Url("http",
+                                                    host="google.com",
+                                                    path='/%EF%BC%AE%EF%BC%AE/abc')),
+
+        # Scheme without ://
+        ("javascript:a='@google.com:12345/';alert(0)",
+         Url(scheme="javascript",
+             path="a='@google.com:12345/';alert(0)")),
+
+        ("//google.com/a/b/c", Url(host="google.com", path="/a/b/c")),
+
+        # International URLs
+        (u'http://ヒ:キ@ヒ.abc.ニ/ヒ?キ#ワ', Url(u'http',
+                                          host=u'xn--pdk.abc.xn--idk',
+                                          auth=u'%E3%83%92:%E3%82%AD',
+                                          path=u'/%E3%83%92',
+                                          query=u'%E3%82%AD',
+                                          fragment=u'%E3%83%AF')),
+
+        # Injected headers (CVE-2016-5699, CVE-2019-9740, CVE-2019-9947)
+        ("10.251.0.83:7777?a=1 HTTP/1.1\r\nX-injected: header",
+         Url(host='10.251.0.83', port=7777, path='',
+             query='a=1%20HTTP/1.1%0D%0AX-injected:%20header')),
+
+        ("http://127.0.0.1:6379?\r\nSET test failure12\r\n:8080/test/?test=a",
+         Url(scheme='http', host='127.0.0.1', port=6379, path='',
+             query='%0D%0ASET%20test%20failure12%0D%0A:8080/test/?test=a')),
+    ]
+
+    @pytest.mark.parametrize("url, expected_url", url_vulnerabilities)
+    def test_url_vulnerabilities(self, url, expected_url):
+        if expected_url is False:
+            with pytest.raises(LocationParseError):
+                parse_url(url)
+        else:
+            assert parse_url(url) == expected_url
+
+    @onlyPy2
+    def test_parse_url_bytes_to_str_python_2(self):
+        url = parse_url(b"https://www.google.com/")
+        assert url == Url('https', host='www.google.com', path='/')
+
+        assert isinstance(url.scheme, str)
+        assert isinstance(url.host, str)
+        assert isinstance(url.path, str)
+
+    @onlyPy2
+    def test_parse_url_unicode_python_2(self):
+        url = parse_url(u"https://www.google.com/")
+        assert url == Url(u'https', host=u'www.google.com', path=u'/')
+
+        assert isinstance(url.scheme, six.text_type)
+        assert isinstance(url.host, six.text_type)
+        assert isinstance(url.path, six.text_type)
+
+    @onlyPy3
+    def test_parse_url_bytes_type_error_python_3(self):
+        with pytest.raises(TypeError):
+            parse_url(b"https://www.google.com/")
+
     @pytest.mark.parametrize('kwargs, expected', [
-        ({'accept_encoding': True},
-         {'accept-encoding': 'gzip,deflate'}),
+        pytest.param(
+            {'accept_encoding': True},
+            {'accept-encoding': 'gzip,deflate,br'},
+            marks=onlyBrotlipy(),
+        ),
+        pytest.param(
+            {'accept_encoding': True},
+            {'accept-encoding': 'gzip,deflate'},
+            marks=notBrotlipy(),
+        ),
         ({'accept_encoding': 'foo,bar'},
          {'accept-encoding': 'foo,bar'}),
         ({'accept_encoding': ['foo', 'bar']},
          {'accept-encoding': 'foo,bar'}),
-        ({'accept_encoding': True, 'user_agent': 'banana'},
-         {'accept-encoding': 'gzip,deflate', 'user-agent': 'banana'}),
+        pytest.param(
+            {'accept_encoding': True, 'user_agent': 'banana'},
+            {'accept-encoding': 'gzip,deflate,br', 'user-agent': 'banana'},
+            marks=onlyBrotlipy(),
+        ),
+        pytest.param(
+            {'accept_encoding': True, 'user_agent': 'banana'},
+            {'accept-encoding': 'gzip,deflate', 'user-agent': 'banana'},
+            marks=notBrotlipy(),
+        ),
         ({'user_agent': 'banana'},
          {'user-agent': 'banana'}),
         ({'keep_alive': True},
@@ -300,6 +442,7 @@ class TestUtil(object):
         (('abcd', ''),   ('abcd', '', None)),
         (('abcd', 'a'),  ('', 'bcd', 'a')),
         (('abcd', 'ab'), ('', 'bcd', 'a')),
+        (('abcd', 'eb'), ('a', 'cd', 'b')),
     ])
     def test_split_first(self, input, expected):
         output = split_first(*input)
@@ -405,7 +548,7 @@ class TestUtil(object):
         assert timeout.get_connect_duration() == 37
 
     @pytest.mark.parametrize('candidate, requirements', [
-        (None, ssl.CERT_NONE),
+        (None, ssl.CERT_REQUIRED),
         (ssl.CERT_NONE, ssl.CERT_NONE),
         (ssl.CERT_REQUIRED, ssl.CERT_REQUIRED),
         ('REQUIRED', ssl.CERT_REQUIRED),
@@ -540,6 +683,13 @@ class TestUtil(object):
                 instance.bind.return_value = True
                 assert _has_ipv6('::1')
 
+    def test_has_ipv6_disabled_on_appengine(self):
+        gae_patch = patch(
+            'urllib3.contrib._appengine_environ.is_appengine_sandbox',
+            return_value=True)
+        with gae_patch:
+            assert not _has_ipv6('::1')
+
     def test_ip_family_ipv6_enabled(self):
         with patch('urllib3.util.connection.HAS_IPV6', True):
             assert allowed_gai_family() == socket.AF_UNSPEC
