@@ -12,13 +12,16 @@ from urllib3.exceptions import (
 )
 from urllib3.response import httplib
 from urllib3.util.ssl_ import HAS_SNI
+from urllib3.util import ssl_
 from urllib3.util.timeout import Timeout
 from urllib3.util.retry import Retry
-from urllib3._collections import HTTPHeaderDict, OrderedDict
+from urllib3._collections import HTTPHeaderDict
 
 from dummyserver.testcase import SocketDummyServerTestCase, consume_socket
 from dummyserver.server import (
-    DEFAULT_CERTS, DEFAULT_CA, COMBINED_CERT_AND_KEY, get_unreachable_address)
+    DEFAULT_CERTS, DEFAULT_CA, COMBINED_CERT_AND_KEY,
+    PASSWORD_KEYFILE, get_unreachable_address
+)
 
 from .. import onlyPy3, LogRecorder
 
@@ -27,6 +30,7 @@ try:
 except ImportError:
     class MimeToolMessage(object):
         pass
+from collections import OrderedDict
 from threading import Event
 import select
 import socket
@@ -34,6 +38,8 @@ import ssl
 
 import pytest
 
+from test import fails_on_travis_gce, requires_ssl_context_keyfile_password
+
 
 class TestCookies(SocketDummyServerTestCase):
 
@@ -81,8 +87,8 @@ class TestSNI(SocketDummyServerTestCase):
         except MaxRetryError:  # We are violating the protocol
             pass
         done_receiving.wait()
-        self.assertTrue(self.host.encode('ascii') in self.buf,
-                        "missing hostname in SSL handshake")
+        self.assertIn(self.host.encode('ascii'), self.buf,
+                      "missing hostname in SSL handshake")
 
 
 class TestClientCerts(SocketDummyServerTestCase):
@@ -228,6 +234,80 @@ class TestClientCerts(SocketDummyServerTestCase):
                 "certificates"
             )
 
+    @requires_ssl_context_keyfile_password
+    def test_client_cert_with_string_password(self):
+        self.run_client_cert_with_password_test(u"letmein")
+
+    @requires_ssl_context_keyfile_password
+    def test_client_cert_with_bytes_password(self):
+        self.run_client_cert_with_password_test(b"letmein")
+
+    def run_client_cert_with_password_test(self, password):
+        """
+        Tests client certificate password functionality
+        """
+        done_receiving = Event()
+        client_certs = []
+
+        def socket_handler(listener):
+            sock = listener.accept()[0]
+            sock = self._wrap_in_ssl(sock)
+
+            client_certs.append(sock.getpeercert())
+
+            data = b''
+            while not data.endswith(b'\r\n\r\n'):
+                data += sock.recv(8192)
+
+            sock.sendall(
+                b'HTTP/1.1 200 OK\r\n'
+                b'Server: testsocket\r\n'
+                b'Connection: close\r\n'
+                b'Content-Length: 6\r\n'
+                b'\r\n'
+                b'Valid!'
+            )
+
+            done_receiving.wait(5)
+            sock.close()
+
+        self._start_server(socket_handler)
+        ssl_context = ssl_.SSLContext(ssl_.PROTOCOL_SSLv23)
+        ssl_context.load_cert_chain(
+            certfile=DEFAULT_CERTS['certfile'],
+            keyfile=PASSWORD_KEYFILE,
+            password=password
+        )
+
+        pool = HTTPSConnectionPool(
+            self.host,
+            self.port,
+            ssl_context=ssl_context,
+            cert_reqs='REQUIRED',
+            ca_certs=DEFAULT_CA,
+        )
+        self.addCleanup(pool.close)
+        pool.request('GET', '/', retries=0)
+        done_receiving.set()
+
+        self.assertEqual(len(client_certs), 1)
+
+    @requires_ssl_context_keyfile_password
+    def test_load_keyfile_with_invalid_password(self):
+        context = ssl_.SSLContext(ssl_.PROTOCOL_SSLv23)
+
+        # Different error is raised depending on context.
+        if ssl_.IS_PYOPENSSL:
+            from OpenSSL.SSL import Error
+            expected_error = Error
+        else:
+            expected_error = ssl.SSLError
+
+        with pytest.raises(expected_error):
+            context.load_cert_chain(certfile=DEFAULT_CERTS["certfile"],
+                                    keyfile=PASSWORD_KEYFILE,
+                                    password=b'letmei')
+
 
 class TestSocketClosing(SocketDummyServerTestCase):
 
@@ -291,7 +371,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
 
         self._start_server(socket_handler)
         http = HTTPConnectionPool(self.host, self.port,
-                                  timeout=0.001,
+                                  timeout=0.01,
                                   retries=False,
                                   maxsize=3,
                                   block=True)
@@ -314,7 +394,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
             sock.close()
 
         self._start_server(socket_handler)
-        pool = HTTPConnectionPool(self.host, self.port, timeout=0.001, retries=True)
+        pool = HTTPConnectionPool(self.host, self.port, timeout=0.01, retries=True)
         self.addCleanup(pool.close)
 
         try:
@@ -335,7 +415,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
             sock.close()
 
         self._start_server(socket_handler)
-        pool = HTTPSConnectionPool(self.host, self.port, timeout=0.001, retries=False)
+        pool = HTTPSConnectionPool(self.host, self.port, timeout=0.01, retries=False)
         self.addCleanup(pool.close)
         try:
             self.assertRaises(ReadTimeoutError, pool.request, 'GET', '/')
@@ -374,7 +454,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
 
         try:
             self._start_server(socket_handler)
-            t = Timeout(connect=0.001, read=0.001)
+            t = Timeout(connect=0.001, read=0.01)
             pool = HTTPConnectionPool(self.host, self.port, timeout=t)
             self.addCleanup(pool.close)
 
@@ -407,7 +487,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
         self.addCleanup(pool.close)
 
         response = pool.urlopen('GET', '/', retries=0, preload_content=False,
-                                timeout=Timeout(connect=1, read=0.001))
+                                timeout=Timeout(connect=1, read=0.01))
         try:
             self.assertRaises(ReadTimeoutError, response.read)
         finally:
@@ -437,7 +517,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
         try:
             self.assertRaises(ReadTimeoutError, pool.urlopen,
                               'GET', '/', retries=False,
-                              timeout=Timeout(connect=1, read=0.001))
+                              timeout=Timeout(connect=1, read=0.01))
         finally:
             timed_out.set()
 
@@ -534,7 +614,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
         with HTTPConnectionPool(self.host, self.port) as pool:
             poolsize = pool.pool.qsize()
             response = pool.urlopen('GET', '/', retries=0, preload_content=False,
-                                    timeout=Timeout(connect=1, read=0.001))
+                                    timeout=Timeout(connect=1, read=0.01))
             try:
                 self.assertRaises(ReadTimeoutError, response.read)
                 self.assertEqual(poolsize, pool.pool.qsize())
@@ -623,7 +703,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
             # First request should fail.
             response = pool.urlopen('GET', '/', retries=0,
                                     preload_content=False,
-                                    timeout=Timeout(connect=1, read=0.001))
+                                    timeout=Timeout(connect=1, read=0.1))
             try:
                 self.assertRaises(ReadTimeoutError, response.read)
             finally:
@@ -632,19 +712,12 @@ class TestSocketClosing(SocketDummyServerTestCase):
             # Second should succeed.
             response = pool.urlopen('GET', '/', retries=0,
                                     preload_content=False,
-                                    timeout=Timeout(connect=1, read=0.1))
+                                    timeout=Timeout(connect=1, read=1))
             self.assertEqual(len(response.read()), 8)
 
     def test_closing_response_actually_closes_connection(self):
         done_closing = Event()
         complete = Event()
-        # The insane use of this variable here is to get around the fact that
-        # Python 2.6 does not support returning a value from Event.wait(). This
-        # means we can't tell if an event timed out, so we can't use the timing
-        # out of the 'complete' event to determine the success or failure of
-        # the test. Python 2 also doesn't have the nonlocal statement, so we
-        # can't write directly to this variable, only mutate it. Hence: list.
-        successful = []
 
         def socket_handler(listener):
             sock = listener.accept()[0]
@@ -666,7 +739,6 @@ class TestSocketClosing(SocketDummyServerTestCase):
             sock.settimeout(1)
             new_data = sock.recv(65536)
             self.assertFalse(new_data)
-            successful.append(True)
             sock.close()
             complete.set()
 
@@ -679,7 +751,7 @@ class TestSocketClosing(SocketDummyServerTestCase):
         response.close()
 
         done_closing.set()  # wait until the socket in our pool gets closed
-        complete.wait(timeout=1)
+        successful = complete.wait(timeout=1)
         if not successful:
             self.fail("Timed out waiting for connection close")
 
@@ -731,18 +803,18 @@ class TestSocketClosing(SocketDummyServerTestCase):
             # save it.
             response = pool.urlopen('GET', '/', retries=1,
                                     release_conn=False, preload_content=False,
-                                    timeout=Timeout(connect=1, read=0.001))
+                                    timeout=Timeout(connect=1, read=0.01))
 
             # The connection should still be on the response object, and none
             # should be in the pool. We opened two though.
             self.assertEqual(pool.num_connections, 2)
             self.assertEqual(pool.pool.qsize(), 0)
-            self.assertTrue(response.connection is not None)
+            self.assertIsNotNone(response.connection)
 
             # Consume the data. This should put the connection back.
             response.read()
             self.assertEqual(pool.pool.qsize(), 1)
-            self.assertTrue(response.connection is None)
+            self.assertIsNone(response.connection)
 
 
 class TestProxyManager(SocketDummyServerTestCase):
@@ -814,7 +886,7 @@ class TestProxyManager(SocketDummyServerTestCase):
         # FIXME: The order of the headers is not predictable right now. We
         # should fix that someday (maybe when we migrate to
         # OrderedDict/MultiDict).
-        self.assertTrue(b'For The Proxy: YEAH!\r\n' in r.data)
+        self.assertIn(b'For The Proxy: YEAH!\r\n', r.data)
 
     def test_retries(self):
         close_event = Event()
@@ -900,7 +972,7 @@ class TestProxyManager(SocketDummyServerTestCase):
         self._start_server(echo_socket_handler)
         base_url = 'http://%s:%d' % (self.host, self.port)
 
-        proxy = proxy_from_url(base_url)
+        proxy = proxy_from_url(base_url, ca_certs=DEFAULT_CA)
         self.addCleanup(proxy.clear)
 
         url = 'https://{0}'.format(self.host)
@@ -944,7 +1016,7 @@ class TestProxyManager(SocketDummyServerTestCase):
         self._start_server(echo_socket_handler)
         base_url = 'http://%s:%d' % (self.host, self.port)
 
-        proxy = proxy_from_url(base_url)
+        proxy = proxy_from_url(base_url, cert_reqs='NONE')
         self.addCleanup(proxy.clear)
 
         url = 'https://[{0}]'.format(ipv6_addr)
@@ -998,8 +1070,7 @@ class TestSSL(SocketDummyServerTestCase):
             ssl_sock = ssl.wrap_socket(sock,
                                        server_side=True,
                                        keyfile=DEFAULT_CERTS['keyfile'],
-                                       certfile=DEFAULT_CERTS['certfile'],
-                                       ca_certs=DEFAULT_CA)
+                                       certfile=DEFAULT_CERTS['certfile'])
 
             buf = b''
             while not buf.endswith(b'\r\n\r\n'):
@@ -1018,11 +1089,11 @@ class TestSSL(SocketDummyServerTestCase):
             ssl_sock.close()
 
         self._start_server(socket_handler)
-        pool = HTTPSConnectionPool(self.host, self.port)
+        pool = HTTPSConnectionPool(self.host, self.port, ca_certs=DEFAULT_CA)
         self.addCleanup(pool.close)
 
         response = pool.urlopen('GET', '/', retries=0, preload_content=False,
-                                timeout=Timeout(connect=1, read=0.001))
+                                timeout=Timeout(connect=1, read=0.01))
         try:
             self.assertRaises(ReadTimeoutError, response.read)
         finally:
@@ -1056,7 +1127,7 @@ class TestSSL(SocketDummyServerTestCase):
                                        assert_fingerprint=fingerprint)
             try:
                 response = pool.urlopen('GET', '/', preload_content=False,
-                                        timeout=Timeout(connect=1, read=0.001),
+                                        timeout=Timeout(connect=1, read=0.01),
                                         retries=0)
                 response.read()
             finally:
@@ -1108,7 +1179,7 @@ class TestSSL(SocketDummyServerTestCase):
 
         self._start_server(socket_handler)
 
-        pool = HTTPSConnectionPool(self.host, self.port)
+        pool = HTTPSConnectionPool(self.host, self.port, ca_certs=DEFAULT_CA)
         self.addCleanup(pool.close)
         response = pool.urlopen('GET', '/', retries=1)
         self.assertEqual(response.data, b'Success')
@@ -1225,6 +1296,7 @@ class TestHeaders(SocketDummyServerTestCase):
         pool.request('GET', '/', headers=OrderedDict(expected_request_headers))
         self.assertEqual(expected_request_headers, actual_request_headers)
 
+    @fails_on_travis_gce
     def test_request_host_header_ignores_fqdn_dot(self):
 
         received_headers = []
@@ -1296,12 +1368,12 @@ class TestHeaders(SocketDummyServerTestCase):
 )
 class TestBrokenHeaders(SocketDummyServerTestCase):
 
-    def _test_broken_header_parsing(self, headers):
+    def _test_broken_header_parsing(self, headers, unparsed_data_check=None):
         self.start_response_handler((
            b'HTTP/1.1 200 OK\r\n'
            b'Content-Length: 0\r\n'
            b'Content-type: text/plain\r\n'
-           ) + b'\r\n'.join(headers) + b'\r\n'
+           ) + b'\r\n'.join(headers) + b'\r\n\r\n'
         )
 
         pool = HTTPConnectionPool(self.host, self.port, retries=False)
@@ -1313,26 +1385,52 @@ class TestBrokenHeaders(SocketDummyServerTestCase):
         for record in logs:
             if 'Failed to parse headers' in record.msg and \
                     pool._absolute_url('/') == record.args[0]:
-                return
+                if unparsed_data_check is None or unparsed_data_check in record.getMessage():
+                    return
         self.fail('Missing log about unparsed headers')
 
     def test_header_without_name(self):
         self._test_broken_header_parsing([
-            b': Value\r\n',
-            b'Another: Header\r\n',
+            b': Value',
+            b'Another: Header',
         ])
 
     def test_header_without_name_or_value(self):
         self._test_broken_header_parsing([
-            b':\r\n',
-            b'Another: Header\r\n',
+            b':',
+            b'Another: Header',
         ])
 
     def test_header_without_colon_or_value(self):
         self._test_broken_header_parsing([
             b'Broken Header',
             b'Another: Header',
-        ])
+        ], 'Broken Header')
+
+
+class TestHeaderParsingContentType(SocketDummyServerTestCase):
+
+    def _test_okay_header_parsing(self, header):
+        self.start_response_handler((
+           b'HTTP/1.1 200 OK\r\n'
+           b'Content-Length: 0\r\n'
+           ) + header + b'\r\n\r\n'
+        )
+
+        pool = HTTPConnectionPool(self.host, self.port, retries=False)
+        self.addCleanup(pool.close)
+
+        with LogRecorder() as logs:
+            pool.request('GET', '/')
+
+        for record in logs:
+            assert 'Failed to parse headers' not in record.msg
+
+    def test_header_text_plain(self):
+        self._test_okay_header_parsing(b'Content-type: text/plain')
+
+    def test_header_message_rfc822(self):
+        self._test_okay_header_parsing(b'Content-type: message/rfc822')
 
 
 class TestHEAD(SocketDummyServerTestCase):
@@ -1433,7 +1531,7 @@ class TestBadContentLength(SocketDummyServerTestCase):
             next(data)
             self.assertFail()
         except ProtocolError as e:
-            self.assertTrue('12 bytes read, 10 more expected' in str(e))
+            self.assertIn('12 bytes read, 10 more expected', str(e))
 
         done_event.set()
 
